# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.source_connection import SourceConnection
from ..types.source_connection_create import SourceConnectionCreate
from ..types.source_connection_list_item import SourceConnectionListItem
from ..types.white_label import WhiteLabel
from .raw_client import AsyncRawWhiteLabelsClient, RawWhiteLabelsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WhiteLabelsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWhiteLabelsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWhiteLabelsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWhiteLabelsClient
        """
        return self._raw_client

    def list_white_labels(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[WhiteLabel]:
        """
        List all white labels for the current user's organization.

        Args:
        -----
            db: The database session
            auth_context: The authentication context

        Returns:
        --------
            list[schemas.WhiteLabel]: A list of white labels

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[WhiteLabel]
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.list_white_labels()
        """
        _response = self._raw_client.list_white_labels(request_options=request_options)
        return _response.data

    def create_white_label(
        self,
        *,
        name: str,
        source_short_name: str,
        redirect_url: str,
        client_id: str,
        client_secret: str,
        allowed_origins: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhiteLabel:
        """
        Create new white label integration.

        Args:
        -----
            db: The database session
            auth_context: The current user
            white_label_in: The white label to create
            logger: The logger with the current authentication context

        Returns:
        --------
            white_label (schemas.WhiteLabel): The created white label

        Parameters
        ----------
        name : str

        source_short_name : str

        redirect_url : str

        client_id : str

        client_secret : str

        allowed_origins : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.create_white_label(
            name="Company Slack Integration",
            source_short_name="slack",
            redirect_url="https://yourapp.com/auth/slack/callback",
            client_id="1234567890.1234567890123",
            client_secret="abcdefghijklmnopqrstuvwxyz123456",
            allowed_origins="https://yourapp.com,https://app.yourapp.com",
        )
        """
        _response = self._raw_client.create_white_label(
            name=name,
            source_short_name=source_short_name,
            redirect_url=redirect_url,
            client_id=client_id,
            client_secret=client_secret,
            allowed_origins=allowed_origins,
            request_options=request_options,
        )
        return _response.data

    def get_white_label(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WhiteLabel:
        """
        Get a specific white label integration.

        Args:
        -----
            db: The database session
            white_label_id: The ID of the white label to get
            auth_context: The authentication context
            logger: The logger with the current authentication context

        Returns:
        --------
            white_label (schemas.WhiteLabel): The white label

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.get_white_label(
            white_label_id="white_label_id",
        )
        """
        _response = self._raw_client.get_white_label(white_label_id, request_options=request_options)
        return _response.data

    def update_white_label(
        self,
        white_label_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        client_secret: typing.Optional[str] = OMIT,
        allowed_origins: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhiteLabel:
        """
        Update a white label integration.

        Args:
        -----
            db: The database session
            auth_context: The authentication context
            white_label_id: The ID of the white label to update
            white_label_in: The white label to update

        Returns:
        --------
            white_label (schemas.WhiteLabel): The updated white label

        Parameters
        ----------
        white_label_id : str

        name : typing.Optional[str]

        redirect_url : typing.Optional[str]

        client_id : typing.Optional[str]

        client_secret : typing.Optional[str]

        allowed_origins : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.update_white_label(
            white_label_id="white_label_id",
        )
        """
        _response = self._raw_client.update_white_label(
            white_label_id,
            name=name,
            redirect_url=redirect_url,
            client_id=client_id,
            client_secret=client_secret,
            allowed_origins=allowed_origins,
            request_options=request_options,
        )
        return _response.data

    def delete_white_label(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WhiteLabel:
        """
        Delete a white label integration.

        Args:
        -----
            db: The database session
            auth_context: The current authentication context
            white_label_id: The ID of the white label to delete

        Returns:
        --------
            white_label (schemas.WhiteLabel): The deleted white label

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.delete_white_label(
            white_label_id="white_label_id",
        )
        """
        _response = self._raw_client.delete_white_label(white_label_id, request_options=request_options)
        return _response.data

    def get_white_label_oauth_2_auth_url(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Generate the OAuth2 authorization URL by delegating to oauth2_service.

        Args:
        -----
            request: The HTTP request
            response: The HTTP response
            db: The database session
            white_label_id: The ID of the white label to get the auth URL for
            auth_context: The current authentication context

        Returns:
        --------
            str: The OAuth2 authorization URL

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.get_white_label_oauth_2_auth_url(
            white_label_id="white_label_id",
        )
        """
        _response = self._raw_client.get_white_label_oauth_2_auth_url(white_label_id, request_options=request_options)
        return _response.data

    def list_white_label_source_connections(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceConnectionListItem]:
        """
        List all source connections for a specific white label.

        Args:
        -----
            white_label_id: The ID of the white label to list source connections for
            db: The database session
            auth_context: The authentication context

        Returns:
        --------
            list[schemas.SourceConnectionListItem]: A list of source connections

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionListItem]
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.list_white_label_source_connections(
            white_label_id="white_label_id",
        )
        """
        _response = self._raw_client.list_white_label_source_connections(
            white_label_id, request_options=request_options
        )
        return _response.data

    def exchange_white_label_oauth_2_code_white_labels_white_label_id_oauth_2_code_options(
        self,
        white_label_id: str,
        *,
        code: str,
        source_connection_in: typing.Optional[SourceConnectionCreate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnection:
        """
        Exchange OAuth2 code for tokens and create connection with source connection.

        Args:
        -----
            request: The HTTP request
            response: The HTTP response
            white_label_id: The ID of the white label to exchange the code for
            code: The OAuth2 code
            source_connection_in: Optional source connection configuration
            db: The database session
            auth_context: The authentication context
            logger: The logger with the current authentication context
            background_tasks: Background tasks for async operations

        Returns:
        --------
            source_connection (schemas.SourceConnection): The created source connection

        Parameters
        ----------
        white_label_id : str

        code : str

        source_connection_in : typing.Optional[SourceConnectionCreate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )
        client.white_labels.exchange_white_label_oauth_2_code_white_labels_white_label_id_oauth_2_code_options(
            white_label_id="white_label_id",
            code="code",
        )
        """
        _response = self._raw_client.exchange_white_label_oauth_2_code_white_labels_white_label_id_oauth_2_code_options(
            white_label_id, code=code, source_connection_in=source_connection_in, request_options=request_options
        )
        return _response.data


class AsyncWhiteLabelsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWhiteLabelsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWhiteLabelsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWhiteLabelsClient
        """
        return self._raw_client

    async def list_white_labels(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[WhiteLabel]:
        """
        List all white labels for the current user's organization.

        Args:
        -----
            db: The database session
            auth_context: The authentication context

        Returns:
        --------
            list[schemas.WhiteLabel]: A list of white labels

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[WhiteLabel]
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.list_white_labels()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_white_labels(request_options=request_options)
        return _response.data

    async def create_white_label(
        self,
        *,
        name: str,
        source_short_name: str,
        redirect_url: str,
        client_id: str,
        client_secret: str,
        allowed_origins: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhiteLabel:
        """
        Create new white label integration.

        Args:
        -----
            db: The database session
            auth_context: The current user
            white_label_in: The white label to create
            logger: The logger with the current authentication context

        Returns:
        --------
            white_label (schemas.WhiteLabel): The created white label

        Parameters
        ----------
        name : str

        source_short_name : str

        redirect_url : str

        client_id : str

        client_secret : str

        allowed_origins : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.create_white_label(
                name="Company Slack Integration",
                source_short_name="slack",
                redirect_url="https://yourapp.com/auth/slack/callback",
                client_id="1234567890.1234567890123",
                client_secret="abcdefghijklmnopqrstuvwxyz123456",
                allowed_origins="https://yourapp.com,https://app.yourapp.com",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_white_label(
            name=name,
            source_short_name=source_short_name,
            redirect_url=redirect_url,
            client_id=client_id,
            client_secret=client_secret,
            allowed_origins=allowed_origins,
            request_options=request_options,
        )
        return _response.data

    async def get_white_label(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WhiteLabel:
        """
        Get a specific white label integration.

        Args:
        -----
            db: The database session
            white_label_id: The ID of the white label to get
            auth_context: The authentication context
            logger: The logger with the current authentication context

        Returns:
        --------
            white_label (schemas.WhiteLabel): The white label

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.get_white_label(
                white_label_id="white_label_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_white_label(white_label_id, request_options=request_options)
        return _response.data

    async def update_white_label(
        self,
        white_label_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        client_secret: typing.Optional[str] = OMIT,
        allowed_origins: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhiteLabel:
        """
        Update a white label integration.

        Args:
        -----
            db: The database session
            auth_context: The authentication context
            white_label_id: The ID of the white label to update
            white_label_in: The white label to update

        Returns:
        --------
            white_label (schemas.WhiteLabel): The updated white label

        Parameters
        ----------
        white_label_id : str

        name : typing.Optional[str]

        redirect_url : typing.Optional[str]

        client_id : typing.Optional[str]

        client_secret : typing.Optional[str]

        allowed_origins : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.update_white_label(
                white_label_id="white_label_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_white_label(
            white_label_id,
            name=name,
            redirect_url=redirect_url,
            client_id=client_id,
            client_secret=client_secret,
            allowed_origins=allowed_origins,
            request_options=request_options,
        )
        return _response.data

    async def delete_white_label(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WhiteLabel:
        """
        Delete a white label integration.

        Args:
        -----
            db: The database session
            auth_context: The current authentication context
            white_label_id: The ID of the white label to delete

        Returns:
        --------
            white_label (schemas.WhiteLabel): The deleted white label

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhiteLabel
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.delete_white_label(
                white_label_id="white_label_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_white_label(white_label_id, request_options=request_options)
        return _response.data

    async def get_white_label_oauth_2_auth_url(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Generate the OAuth2 authorization URL by delegating to oauth2_service.

        Args:
        -----
            request: The HTTP request
            response: The HTTP response
            db: The database session
            white_label_id: The ID of the white label to get the auth URL for
            auth_context: The current authentication context

        Returns:
        --------
            str: The OAuth2 authorization URL

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.get_white_label_oauth_2_auth_url(
                white_label_id="white_label_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_white_label_oauth_2_auth_url(
            white_label_id, request_options=request_options
        )
        return _response.data

    async def list_white_label_source_connections(
        self, white_label_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceConnectionListItem]:
        """
        List all source connections for a specific white label.

        Args:
        -----
            white_label_id: The ID of the white label to list source connections for
            db: The database session
            auth_context: The authentication context

        Returns:
        --------
            list[schemas.SourceConnectionListItem]: A list of source connections

        Parameters
        ----------
        white_label_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionListItem]
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.list_white_label_source_connections(
                white_label_id="white_label_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_white_label_source_connections(
            white_label_id, request_options=request_options
        )
        return _response.data

    async def exchange_white_label_oauth_2_code_white_labels_white_label_id_oauth_2_code_options(
        self,
        white_label_id: str,
        *,
        code: str,
        source_connection_in: typing.Optional[SourceConnectionCreate] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SourceConnection:
        """
        Exchange OAuth2 code for tokens and create connection with source connection.

        Args:
        -----
            request: The HTTP request
            response: The HTTP response
            white_label_id: The ID of the white label to exchange the code for
            code: The OAuth2 code
            source_connection_in: Optional source connection configuration
            db: The database session
            auth_context: The authentication context
            logger: The logger with the current authentication context
            background_tasks: Background tasks for async operations

        Returns:
        --------
            source_connection (schemas.SourceConnection): The created source connection

        Parameters
        ----------
        white_label_id : str

        code : str

        source_connection_in : typing.Optional[SourceConnectionCreate]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SourceConnection
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
            organization_id="YOUR_ORGANIZATION_ID",
        )


        async def main() -> None:
            await client.white_labels.exchange_white_label_oauth_2_code_white_labels_white_label_id_oauth_2_code_options(
                white_label_id="white_label_id",
                code="code",
            )


        asyncio.run(main())
        """
        _response = (
            await self._raw_client.exchange_white_label_oauth_2_code_white_labels_white_label_id_oauth_2_code_options(
                white_label_id, code=code, source_connection_in=source_connection_in, request_options=request_options
            )
        )
        return _response.data
